import React, { useState, useEffect, useRef, useCallback } from "react";
import { useTheme } from "../hooks/useTheme";
import { tokenize } from "../utils/tokenizer";
import type { HighlighterProps, Token } from "../types";
import { 
  Copy, 
  Check, 
  ChevronDown, 
  ChevronUp, 
  Code,
  Search,
  Sun,
  Moon,
  Share2,
  Download
} from "lucide-react";

export const SyntaxHighlighter: React.FC<HighlighterProps> = ({
  code,
  language,
  showLineNumbers = false,
  wrapLines = false,
  maxHeight = "400px",
  isDarkMode,
  expandable = false,
  className = "",
  style = {},
  customBackgroundColor,
  showCopyButton = true,
  fileName = "",
  showSearch = false,
  showShare = false,
  showThemeToggle = false,
  showDownload = false,
}) => {
  const { theme, isDarkMode: contextDarkMode } = useTheme();
  const [copied, setCopied] = useState(false);
  const [isExpanded, setIsExpanded] = useState(false);
  const [searchTerm, setSearchTerm] = useState("");
  const [searchResults, setSearchResults] = useState<number[]>([]);
  const [currentSearchIndex, setCurrentSearchIndex] = useState(-1);
  const [codeHeight, setCodeHeight] = useState<number>(0);
  const codeRef = useRef<HTMLPreElement>(null);
  const currentDarkMode = isDarkMode ?? contextDarkMode;

  // Measure code height on mount and when code changes
  useEffect(() => {
    if (codeRef.current) {
      setCodeHeight(codeRef.current.scrollHeight);
    }
  }, [code]);

  const copyToClipboard = async () => {
    try {
      await navigator.clipboard.writeText(code);
      setCopied(true);
      setTimeout(() => setCopied(false), 2000);
    } catch (err) {
      console.error("Failed to copy:", err);
    }
  };

  const handleSearch = useCallback(() => {
    if (!searchTerm) {
      setSearchResults([]);
      setCurrentSearchIndex(-1);
      return;
    }

    const results = code.split("\n").reduce((acc: number[], line, index) => {
      if (line.toLowerCase().includes(searchTerm.toLowerCase())) {
        acc.push(index);
      }
      return acc;
    }, []);

    setSearchResults(results);
    setCurrentSearchIndex(results.length > 0 ? 0 : -1);
  }, [searchTerm, code]);

  useEffect(() => {
    handleSearch();
  }, [handleSearch]);

  const navigateSearch = (direction: "next" | "prev") => {
    if (searchResults.length === 0) return;
    const newIndex = direction === "next"
      ? (currentSearchIndex + 1) % searchResults.length
      : (currentSearchIndex - 1 + searchResults.length) % searchResults.length;
    setCurrentSearchIndex(newIndex);

    // Scroll to the highlighted line
    if (codeRef.current) {
      const lines = codeRef.current.querySelectorAll('[data-line-number]');
      const targetLine = lines[searchResults[newIndex]];
      if (targetLine) {
        targetLine.scrollIntoView({ behavior: 'smooth', block: 'center' });
      }
    }
  };

  const shareCode = async () => {
    try {
      await navigator.share({
        title: `Code Snippet - ${language}`,
        text: code,
        url: window.location.href,
      });
    } catch (err) {
      console.error("Failed to share:", err);
    }
  };

  const downloadCode = () => {
    const element = document.createElement("a");
    const file = new Blob([code], { type: "text/plain" });
    element.href = URL.createObjectURL(file);
    element.download = fileName || `code.${language}`;
    document.body.appendChild(element);
    element.click();
    document.body.removeChild(element);
  };

  const getTokenClassName = (type: string): string => {
    const colors = {
      keyword: "text-rose-500 dark:text-rose-400 font-medium",
      string: "text-emerald-600 dark:text-emerald-400",
      number: "text-blue-600 dark:text-blue-400 font-medium",
      boolean: "text-purple-600 dark:text-purple-400 font-medium",
      comment: "text-slate-500 dark:text-slate-400 italic",
      operator: "text-amber-600 dark:text-amber-400 font-medium",
      punctuation: "text-slate-600 dark:text-slate-400",
      function: "text-violet-600 dark:text-violet-400 font-medium",
      class: "text-cyan-600 dark:text-cyan-400 font-medium",
      property: "text-fuchsia-600 dark:text-fuchsia-400",
      variable: "text-orange-600 dark:text-orange-400",
      builtin: "text-teal-600 dark:text-teal-400 font-medium",
      decorator: "text-yellow-600 dark:text-yellow-400",
      regex: "text-red-600 dark:text-red-400",
      default: "text-slate-900 dark:text-slate-100"
    };

    return colors[type as keyof typeof colors] || colors.default;
  };

  const bgColor = customBackgroundColor || (currentDarkMode ? "bg-slate-900" : "bg-slate-50");

  const codeContainerStyle = {
    maxHeight: isExpanded ? `${codeHeight}px` : maxHeight,
    transition: 'max-height 0.3s ease-out',
    overflow: 'hidden'
  };

  return (
    <div className={`rounded-lg overflow-hidden border border-gray-200 dark:border-gray-800 ${className}`}>
      <div className="flex items-center justify-between px-4 py-2  dark:bg-gray-800 border-b border-gray-200 dark:border-gray-700">
        <div className="flex items-center space-x-2">
          <Code className="w-4 h-4 text-gray-500 dark:text-gray-400" />
          {fileName && (
            <span className="text-sm font-medium text-gray-700 dark:text-gray-300">
              {fileName}
            </span>
          )}
        </div>
        <div className="flex items-center space-x-2">
          {showThemeToggle && (
            <button onClick={() => document.documentElement.classList.toggle("dark")}
                    className="p-1 rounded hover:bg-gray-100 dark:hover:bg-gray-700">
              {currentDarkMode ? <Moon className="w-4 h-4" /> : <Sun className="w-4 h-4" />}
            </button>
          )}
          {showShare && (
            <button onClick={shareCode}
                    className="p-1 rounded hover:bg-gray-100 dark:hover:bg-gray-700">
              <Share2 className="w-4 h-4" />
            </button>
          )}
          {showDownload && (
            <button onClick={downloadCode}
                    className="p-1 rounded hover:bg-gray-100 dark:hover:bg-gray-700">
              <Download className="w-4 h-4" />
            </button>
          )}
          {showCopyButton && (
            <button onClick={copyToClipboard}
                    className="p-1 rounded hover:bg-gray-100 dark:hover:bg-gray-700">
              {copied ? (
                <Check className="w-4 h-4 text-green-500" />
              ) : (
                <Copy className="w-4 h-4 text-gray-500 dark:text-gray-400" />
              )}
            </button>
          )}
        </div>
      </div>

      {showSearch && (
        <div className="p-4 bg-white dark:bg-gray-800 border-b border-gray-200 dark:border-gray-700">
          <div className="flex items-center space-x-2">
            <Search className="w-4 h-4 text-gray-400" />
            <input
              type="text"
              value={searchTerm}
              onChange={(e) => setSearchTerm(e.target.value)}
              placeholder="Search in code..."
              className="flex-1 px-3 py-1 text-sm bg-gray-50 dark:bg-gray-700 border border-gray-200 dark:border-gray-600 rounded"
            />
            {searchTerm && (
              <div className="flex items-center space-x-2">
                <button onClick={() => navigateSearch("prev")}
                        className="px-2 py-1 text-sm rounded bg-gray-100 dark:bg-gray-700">
                  ↑
                </button>
                <button onClick={() => navigateSearch("next")}
                        className="px-2 py-1 text-sm rounded bg-gray-100 dark:bg-gray-700">
                  ↓
                </button>
                <span className="text-sm text-gray-500">
                  {searchResults.length > 0 ? `${currentSearchIndex + 1}/${searchResults.length}` : "No results"}
                </span>
              </div>
            )}
          </div>
        </div>
      )}

      <div className={`relative ${bgColor}`} style={codeContainerStyle}>
        <pre
          ref={codeRef}
          className={`p-4 text-sm font-mono overflow-x-auto ${wrapLines ? "whitespace-pre-wrap" : "whitespace-pre"}`}
          style={style}
        >
          <code>
            {code.split("\n").map((line, lineIndex) => (
              <div 
                key={lineIndex}
                data-line-number={lineIndex} 
                className={`flex ${searchResults.includes(lineIndex) ? "bg-yellow-100/20" : ""}`}
              >
                {showLineNumbers && (
                  <span className="select-none mr-4 text-gray-400 dark:text-gray-500 text-right w-8 flex-none">
                    {lineIndex + 1}
                  </span>
                )}
                <span className="flex-1">
                  {tokenize(line, language).map((token, tokenIndex) => (
                    <span
                      key={tokenIndex}
                      className={getTokenClassName(token.type)}
                    >
                      {token.content}
                    </span>
                  ))}
                </span>
              </div>
            ))}
          </code>
        </pre>

        {!isExpanded && (
          <div className="absolute bottom-0 left-0 right-0 h-24 bg-gradient-to-t from-slate-50 dark:from-slate-900 to-transparent pointer-events-none" />
        )}
      </div>

      {expandable && (
        <button
          onClick={() => setIsExpanded(!isExpanded)}
          className="w-full px-4 py-2 text-sm text-gray-700 dark:text-gray-300 bg-white dark:bg-gray-800 hover:bg-gray-50 dark:hover:bg-gray-700 flex items-center justify-center gap-2 transition-colors border-t border-gray-200 dark:border-gray-700"
        >
          {isExpanded ? (
            <>
              <ChevronUp className="w-4 h-4" />
              Show Less
            </>
          ) : (
            <>
              <ChevronDown className="w-4 h-4" />
              Show More
            </>
          )}
        </button>
      )}
    </div>
  );
};

export default SyntaxHighlighter;
